local buttonValues = {BT_WEAPONNEXT, BT_WEAPONPREV, BT_FIRENORMAL, BT_TOSSFLAG, BT_CUSTOM1, BT_CUSTOM2, BT_CUSTOM3}

local skinTable = {}
local skinToNum = {}
for skin in skins.iterate
	skinTable[#skin] = skin.name
	skinToNum[skin.name] = #skin
end
addHook("MapLoad", do
	skinTable = {}
	skinToNum = {}
	for skin in skins.iterate
		skinTable[#skin] = skin.name
		skinToNum[skin.name] = #skin
	end
end)

-- Support for custom colors rolling in!
local colTable = {
	[SKINCOLOR_CFIGHTER] = true
}
rawset(_G, "CLONE_ColorTable", colTable)

local function Ready(player, skin)
	if R_SkinUsable(player, skin)
	and not (player.pflags & (PF_SPINNING|PF_STARTDASH|PF_SHIELDABILITY))
	and not (player.pflags & PF_THOKKED and player.mo.skin != "kirby" and player.mo.skin != "metaknight")
	and not player.climbing
	and player.mo.state ~= S_PLAY_CFSWITCH
		return true
	end
	
	return false
end
rawset(_G, "CL_ReadyToSwitch", Ready)

addHook("PlayerSpawn", function(player)
	player.nextCv = $ or 1
	player.prevCv = $ or 2
end)

local function CloneThinker(player)
	if not player.mo or not CLONE_ColorTable[player.skincolor] then
		return
	end
	
	player.cloneFighter = true
	player.mo.color = player.skincolor
	local skincolor = player.skincolor
	
	player.cloneSwitchNext = $ or BT_WEAPONNEXT
	player.cloneSwitchPrev = $ or BT_WEAPONPREV
	
	if player.cloneSwitchNext ~= buttonValues[player.nextCv]
		player.cloneSwitchNext = buttonValues[player.nextCv] or BT_WEAPONNEXT
	end
	if player.cloneSwitchPrev ~= buttonValues[player.prevCv]
		player.cloneSwitchPrev = buttonValues[player.prevCv] or BT_WEAPONPREV
	end
	
	player.cloneSkin = skinToNum[player.mo.skin]
	player.cloneNext = player.cloneSkin + 1
	if player.cloneNext > #skinTable then player.cloneNext = 0 end
	player.clonePrev = player.cloneSkin - 1
	if player.clonePrev < 0 then player.clonePrev = #skinTable end
	
	if player.cmd.buttons & player.cloneSwitchNext
	or player.cmd.buttons & player.cloneSwitchPrev
		if not player.cloneSwitching
			if player.cmd.buttons & player.cloneSwitchNext
			and CL_ReadyToSwitch(player, player.cloneNext)
				R_SetPlayerSkin(player, player.cloneNext)
				player.skincolor = skincolor
				player.mo.color = player.skincolor
				player.cloneTimer = 10
				player.cloneSkin = skinToNum[player.mo.skin]
				S_StartSound(player.mo, sfx_clswtc)
				player.pflags = $ & ~PF_THOKKED
			elseif CL_ReadyToSwitch(player, player.clonePrev)
				R_SetPlayerSkin(player, player.clonePrev)
				player.skincolor = skincolor
				player.mo.color = player.skincolor
				player.cloneTimer = 10
				player.cloneSkin = skinToNum[player.mo.skin]
				S_StartSound(player.mo, sfx_clswtc)
				player.pflags = $ & ~PF_THOKKED
			end
		end
		player.cloneSwitching = true
	else
		player.cloneSwitching = false
	end
	
	if player.cloneTimer
		player.pflags = $|PF_FULLSTASIS
		player.cloneSwitching = true
		player.mo.state = S_PLAY_CFSWITCH
		if player.cloneTimer == 1
			player.mo.state = S_PLAY_JUMP
			S_StartSound(player.mo, sfx_cldone)
		end
		if player.cloneTimer % 3 == 0 then
			local a = P_SpawnMobjFromMobj(player.mo, 0,0,-5*FRACUNIT, MT_VWREF)
			P_SetObjectMomZ(a, 6*FRACUNIT)
			a.fuse = 15
			a = P_SpawnMobjFromMobj(player.mo, 0,0,-5*FRACUNIT, MT_VWREB)
			P_SetObjectMomZ(a, 6*FRACUNIT)
			a.fuse = 15
		end
		player.cloneTimer = $-1
	end
end
addHook("PlayerThink", function(player)
	if player.playerstate == PST_LIVE
		CloneThinker(player)
	end
end)

local function ClonePost(player)
	if not (player.mo and CLONE_ColorTable[player.skincolor])
	and player.playerstate == PST_LIVE
			if player.cloneFighter then
			player.mo.colorized = false -- Reset these
			player.mo.frame = $ & ~(TR_TRANS30|FF_FULLBRIGHT)
			if player.followmobj
				player.followmobj.colorized = false -- Reset these
				player.followmobj.frame = $ & ~(TR_TRANS30|FF_FULLBRIGHT)
			end
			player.cloneSkin = nil
			player.cloneFighter = false
		end
	else
		if player.cloneTimer and player.mo.state == S_PLAY_CFSWITCH
			player.mo.frame = 9-player.cloneTimer
		end
		
		player.mo.colorized = true
		player.mo.frame = TR_TRANS30|FF_FULLBRIGHT|$
		if player.followmobj
			player.followmobj.colorized = true
			player.followmobj.frame = TR_TRANS30|FF_FULLBRIGHT|$
		end
	end
end
addHook("PostThinkFrame", do
	for player in players.iterate()
		ClonePost(player)
	end
end)

local buttonNames = {"Next Weapon","Prev. Weapon","Fire Normal","Toss Flag","Custom 1","Custom 2","Custom 3"}
local function Yellow(check)
	if check
		return V_GRAYMAP
	else
		return V_YELLOWMAP
	end
end

hud.add(function(v, player)
	if not player.cloneFighter
	or not player.nextCv
	or player.cloneNext == nil
	or not skinTable then return end
	
	local swNext = buttonNames[player.nextCv].." - "..skins[player.cloneNext].realname
	local swPrev = buttonNames[player.prevCv].." - "..skins[player.clonePrev].realname
	if swNext and swPrev then
		v.drawString(16, 152, swNext, V_SNAPTOLEFT|V_SNAPTOBOTTOM|Yellow(player.cloneSwitching or not CL_ReadyToSwitch(player, player.cloneNext))|v.localTransFlag(), "thin")
		v.drawString(16, 160, swPrev, V_SNAPTOLEFT|V_SNAPTOBOTTOM|Yellow(player.cloneSwitching or not CL_ReadyToSwitch(player, player.clonePrev))|v.localTransFlag(), "thin")
	end
end)